'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
var WS_READY_STATE_CONNECTING = exports.WS_READY_STATE_CONNECTING = 0;
var WS_READY_STATE_OPEN = exports.WS_READY_STATE_OPEN = 1;
var WS_READY_STATE_CLOSING = exports.WS_READY_STATE_CLOSING = 2;
var WS_READY_STATE_CLOSED = exports.WS_READY_STATE_CLOSED = 3;

var defaultProps = exports.defaultProps = {
  heartbeat: false,
  heartbeatMessage: '---heartbeat---',
  heartbeatInterval: 15000,
  intervalReconnect: 5000,
  autoReconnect: false,
  useMessageAsReduxAction: false
};

var getSocketIndex = exports.getSocketIndex = function getSocketIndex(webSocketList, url) {
  if (!url && webSocketList.length === 1) return 0;
  return webSocketList.findIndex(function (el) {
    return el.url === url;
  });
};

var getSocket = exports.getSocket = function getSocket(webSocketList, url) {
  if (!url && webSocketList.length === 1) return webSocketList[0];
  return webSocketList.find(function (el) {
    return el.url === url;
  });
};

var addSocket = exports.addSocket = function addSocket(webSocketList, ws) {
  var newWebSocketList = Object.assign(webSocketList);

  newWebSocketList.push(ws);

  return newWebSocketList;
};

var removeSocket = exports.removeSocket = function removeSocket(webSocketList, url) {
  var activeSocketIndex = getSocketIndex(webSocketList, url);

  var sliceArray = webSocketList.slice();
  if (activeSocketIndex >= 0) {
    sliceArray.splice(activeSocketIndex, 1);
  }

  return sliceArray;
};

var addSocketToCloseByUser = exports.addSocketToCloseByUser = function addSocketToCloseByUser(closeByUserList, url) {
  var newCloseByUserList = Object.assign(closeByUserList);
  newCloseByUserList.push(url);
  return newCloseByUserList;
};

var removeSocketToCloseByUser = exports.removeSocketToCloseByUser = function removeSocketToCloseByUser(closeByUserList, url) {
  var activeSocketIndex = closeByUserList.findIndex(function (el) {
    return el === url;
  });

  var sliceArray = closeByUserList.slice();
  if (activeSocketIndex >= 0) {
    sliceArray.splice(activeSocketIndex, 1);
  }

  return activeSocketIndex;
};

var checkCloseByUser = exports.checkCloseByUser = function checkCloseByUser(closeByUserList, url) {
  return closeByUserList.findIndex(function (el) {
    return el === url;
  }) !== -1;
};

var stopHeartbeat = exports.stopHeartbeat = function stopHeartbeat(webSocketTimeoutList, webSocket) {
  var newWebSocketTimeoutList = Object.assign(webSocketTimeoutList);

  clearTimeout(newWebSocketTimeoutList[webSocket.url]);
  delete newWebSocketTimeoutList[webSocket.url];

  return newWebSocketTimeoutList;
};

var startHeartbeat = exports.startHeartbeat = function startHeartbeat(webSocketTimeoutList, webSocket) {
  var heartbeatMessage = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : defaultProps.heartbeatMessage;
  var heartbeatInterval = arguments[3];

  var newWebSocketTimeoutList = stopHeartbeat(webSocketTimeoutList, webSocket);

  newWebSocketTimeoutList[webSocket.url] = setTimeout(function () {
    webSocket.send(heartbeatMessage);
    startHeartbeat(webSocketTimeoutList, webSocket, heartbeatMessage, heartbeatInterval);
  }, heartbeatInterval || defaultProps.heartbeatInterval);

  return webSocketTimeoutList;
};

var getCloseReasonDescription = exports.getCloseReasonDescription = function getCloseReasonDescription(event) {
  switch (event.code) {
    case 1000:
      return 'Normal closure, meaning that the purpose for which the connection was established has been fulfilled.';
    case 1001:
      return 'An endpoint is "going away", such as a server going down or a browser having navigated away from a page.';
    case 1002:
      return 'An endpoint is terminating the connection due to a protocol error';
    case 1003:
      return 'An endpoint is terminating the connection because it has received a type of data it cannot accept (e.g., an endpoint that understands only text data MAY send this if it receives a binary message).';
    case 1004:
      return 'Reserved. The specific meaning might be defined in the future.';
    case 1005:
      return 'No status code was actually present.';
    case 1006:
      return 'The connection was closed abnormally, e.g., without sending or receiving a Close control frame';
    case 1007:
      return 'An endpoint is terminating the connection because it has received data within a message that was not consistent with the type of the message (e.g., non-UTF-8 [http://tools.ietf.org/html/rfc3629] data within a text message).';
    case 1008:
      return 'An endpoint is terminating the connection because it has received a message that "violates its policy". This reason is given either if there is no other sutible reason, or if there is a need to hide specific details about the policy.';
    case 1009:
      return 'An endpoint is terminating the connection because it has received a message that is too big for it to process.';
    case 1010:
      return 'An endpoint (client) is terminating the connection because it has expected the server to negotiate one or more extension, but the server didn\'t return them in the response message of the WebSocket handshake. <br /> Specifically, the extensions that are needed are: ' + event.reason;
    case 1011:
      return 'A server is terminating the connection because it encountered an unexpected condition that prevented it from fulfilling the request.';
    case 1015:
      return 'The connection was closed due to a failure to perform a TLS handshake (e.g., the server certificate can\'t be verified).';
    default:
      return 'Unknown reason';
  }
};